@@ -244,6 +244,14 @@
   (reverse seq))
 
 
+(defun adjoin (obj list)
+  (if (member obj list) list (cons obj list)))
+
+
+(defun array-rank (array)
+  (length (array-dimensions array)))
+
+
 (defun array-dimension (array axis)
   (nth axis (array-dimensions array)))
 
@@ -252,6 +260,14 @@
   (apply #'* (array-dimensions array)))
 
 
+(defun array-in-bounds-p (array &rest inds)
+  (and
+   (eql (array-rank array) (length inds))
+   (every (lambda (x y) (and (>= x 0) (< x y)))
+	  inds
+	  (array-dimensions array))))
+
+
 (defun char (str ind)
   (aref str ind))
 
@@ -260,6 +276,80 @@
   (not (string= str1 str2)))
 
 
+(defun char-equal (&rest chars)
+  (apply #'char= (mapcar #'char-upcase chars)))
+
+
+(defun digit-char (weight &optional (radix 10))
+  (if (or (>= weight radix))
+      ()
+      (aref "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ" weight)))
+
+
+(defun digit-char-p (char &optional (radix 10))
+  (let ((str "00112233445566778899aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyYzZ"))
+    (dotimes (i (* 2 radix))
+      (if (char= char (aref str i))
+	  (return-from digit-char-p (values (floor i 2)))))
+    nil))
+
+
+(defun string-upcase (s)
+  (let* ((str (string s))
+	 (ret (make-string (length str))))
+    (dotimes (i (length str))
+      (setf (aref ret i) (char-upcase (aref str i))))
+    ret))
+
+
+(defun string-downcase (s)
+  (let* ((str (string s))
+	 (ret (make-string (length str))))
+    (dotimes (i (length str))
+      (setf (aref ret i) (char-downcase (aref str i))))
+    ret))
+
+
+(defun string-capitalize (s)
+  (let* ((str (string s))
+	 (ret (make-string (length str)))
+	 (notword t))
+    (dotimes (i (length str))
+      (if (alphanumericp (aref str i))
+	  (if notword
+	      (progn
+		(setf (aref ret i) (char-upcase (aref str i)))
+		(setq notword nil))
+	      (setf (aref ret i) (aref str i)))
+	  (progn
+	    (setq notword t)
+	    (setf (aref ret i) (aref str i)))))
+    ret))
+
+
+(defun nstring-upcase (str)
+  (dotimes (i (length str))
+    (setf (aref str i) (char-upcase (aref str i))))
+  str)
+
+
+(defun nstring-downcase (str)
+  (dotimes (i (length str))
+    (setf (aref str i) (char-downcase (aref str i))))
+  str)
+
+
+(defun nstring-capitalize (str)
+  (let ((notword t))
+    (dotimes (i (length str))
+      (if (alphanumericp (aref str i))
+	  (when notword
+	    (setf (aref str i) (char-upcase (aref str i)))
+	    (setq notword nil))
+	  (setq notword t))))
+  str)
+
+
 
 (defun consp (obj)
   (typep obj 'cons))
@@ -313,6 +403,29 @@
   (typep obj 'number))
 
 
+
+(defun equal (x y)
+  (cond
+    ((numberp x) (eql x y))
+    ((characterp x) (eql x y))
+    ((consp x) (and (equal (car x) (car y)) (equal (cdr x) (cdr y))))
+    (t (eq x y))))
+
+
+(defun equalp (x y)
+  (cond
+    ((and (numberp x) (numberp y)) (= x y))
+    ((and (characterp x) (characterp y)) (char-equal x y))
+    ((consp x) (and (equalp (car x) (car y)) (equalp (cdr x) (cdr y))))
+    ((and (arrayp x) (arrayp y)) (and
+				  (equal (array-dimensions x) (array-dimensions y))
+				  (dotimes (i (array-total-size x) t)
+				    (unless (equalp (row-major-aref x i) (row-major-aref y i))
+				      (return-from equalp nil)))))
+    (t (eq x y))))
+
+
+
 (defun fdefinition (fname)
   (symbol-function fname))
 
@@ -338,3 +451,33 @@
   (write-string string)
   (terpri)
   string)
+
+
+(defun prin1 (obj)
+  (let ((*print-escape* t))
+    (write obj)))
+
+
+(defun princ (obj)
+  (let ((*print-escape* nil)
+	(*print-readably* nil))
+    (write obj)))
+
+
+(defun format (out fstr &rest args)
+  (let (in-spec at-sign colon sign num dirargs)
+    (dotimes (i (length fstr))
+      (let ((ch (elt fstr i)))
+	(if in-spec
+	    (cond
+	      ((char= ch #\@) (setq at-sign t))
+	      ((char= ch #\:) (setq colon t))
+	      ((char= ch #\,) (setq dirargs (cons num dirargs) num nil))
+	      ((char= ch #\~) (write-char #\~) (setq in-spec nil))
+	      ((char= ch #\%) (write-char #\newline) (setq in-spec nil))
+	      ((char= ch #\&) (fresh-line) (setq in-spec nil))
+	      ((char-equal ch #\s) (prin1 (car args)) (setq args (cdr args)) (setq in-spec nil))
+	      ((char-equal ch #\a) (princ (car args)) (setq args (cdr args)) (setq in-spec nil)))
+	    (if (char= ch #\~)
+		(setq in-spec t at-sign nil colon nil sign nil num nil dirargs nil)
+		(write-char ch)))))))
