@@ -58,6 +58,7 @@ for ((1..5))
 make_test ("`'`\"\"", "'`\"\"");
 make_test ("'(((\n" .
 	   ")))", "((()))");
+make_test ("'(\"\"`\"\")", "(\"\" `\"\")");
 make_test ("'(\n" .
 	   "(\n" .
 	   ";\n" .
@@ -68,6 +69,9 @@ make_test ("` ( ; asd\n" .
            " \"\")", "(\"\")");
 make_test ("#|\n" .
 	   "|# \"\"", "\"\"");
+make_test ("#|\n" .
+	   "|# #|\n" .
+	   "|# \"\"", "\"\"");
 make_test (" #| #|\n" .
 	   "|##|\n" .
 	   "|#\n" .
@@ -87,6 +91,7 @@ make_test ("'(1 . (\n" .
 make_test ("'( \"\" #||# )", "(\"\")");
 make_test ("'asd\\f|gh|j", "|ASDfghJ|");
 make_test ("'\\\n ", "|\n|");
+make_test ("'a\\\n ", "|A\n|");
 make_test (":\\asd\\\\f", ":|aSD\\\\F|");
 
 
@@ -129,7 +134,11 @@ make_test ("``(,,.(cdr '(0 1 2 3)) ,4)", "`(,1 ,2 ,3 ,4)");
 
 make_test ("#\\a", "#\\a");
 make_test ("#\\κ", "#\\κ");
+make_test ("#\\,", "#\\,");
+make_test ("#\\\\", "#\\\\");
+make_test ("#\\|", "#\\|");
 make_test ("#\\newLINE", "#\\Newline");
+make_test ("#\\\n ", "#\\Newline");
 make_test ("#.\"\"", "\"\"");
 make_test ("#p\".\"", "#P\".\"");
 make_test ("#(1 2 3)", "#(1 2 3)");
@@ -197,6 +206,7 @@ make_test ("(setf b 10)", "10");
 make_test ("(setf b 9)", "9");
 make_test ("(setf c 4)", "4");
 make_test ("c", "4");
+make_test ("(setf)", "()");
 make_test ("(let ((c 0)) c)", "0");
 make_test ("(setf c (+ c 1))", "5");
 make_test ("c", "5");
@@ -207,6 +217,12 @@ make_test ("(setf (car tt) 'f)", "F");
 make_test ("tt", "(F B D)");
 make_test ("(setf (cdr tt) 'g)", "G");
 make_test ("tt", "(F . G)");
+make_test ("(defparameter str \"abcdef\")", "STR");
+make_test ("(setf (aref str 2) #\\z)", "#\\z");
+make_test ("str", "\"abzdef\"");
+make_test ("(defparameter arr (make-array 3))", "ARR");
+make_test ("(setf (aref arr 2) \"aaa\")", "\"aaa\"");
+make_test ("arr", "#(() () \"aaa\")");
 make_test ("(let ((b 10)) b)", "10");
 make_test ("(let ((b 1)) (setf b 2) b)", "2");
 make_test ("b", "9");
@@ -282,11 +298,15 @@ make_test ("(identity \"hi\")", "\"hi\"");
 make_test ("(1+ 5)", "6");
 make_test ("(1- .5)", "-0.5");
 make_test ("(typep '(1 . 2) 'cons)", "T");
+make_test ("(typep () 'atom)", "T");
+make_test ("(typep (cons 1 2) 'atom)", "()");
 make_test ("(typep 0 'number)", "T");
 make_test ("(typep 0.1 'real)", "T");
 make_test ("(typep 0 'rational)", "T");
 make_test ("(typep 0.1 'rational)", "()");
 make_test ("(typep 0 'integer)", "T");
+make_test ("(typep 0 'bignum)", "T");
+make_test ("(typep 0 'fixnum)", "()");
 make_test ("(typep 1/2 'ratio)", "T");
 make_test ("(typep 3/1 'ratio)", "()");
 make_test ("(typep 0.1 'float)", "T");
@@ -308,6 +328,7 @@ make_test ("(typep 'aaa 'symbol)", "T");
 make_test ("(typep nil 'boolean)", "T");
 make_test ("(typep #p\"\" 'pathname)", "T");
 make_test ("(typep (open #p\"README\") 'stream)", "T");
+make_test ("(make-string 3)", "\"\0\0\0\"");
 make_test ("(make-symbol \"aaa\")", "#:|aaa|");
 make_test ("(boundp 'b)", "T");
 make_test ("(boundp 'g3)", "()");
@@ -326,6 +347,14 @@ make_test ("(symbol-name :bbb)", "\"BBB\"");
 make_test ("(special-operator-p 'if)", "T");
 make_test ("(special-operator-p 'car)", "()");
 make_test ("(special-operator-p 'aaa)", "()");
+make_test ("(equal #\\a #\\a)", "T");
+make_test ("(equal 0 0)", "T");
+make_test ("(equal 0 0.0)", "()");
+make_test ("(equal (cons 'a 'b) (cons 'a 'b))", "T");
+make_test ("(equal (make-array '(1 2 3)) (make-array '(1 2 3)))", "()");
+make_test ("(equalp 0 0)", "T");
+make_test ("(equalp 0 0.0)", "T");
+make_test ("(equalp (make-array '(1 2 3)) (make-array '(1 2 3)))", "T");
 make_test ("(string \"ccc\")", "\"ccc\"");
 make_test ("(string 'ddd)", "\"DDD\"");
 make_test ("(string #\\F)", "\"F\"");
@@ -333,6 +362,39 @@ make_test ("(string= \"aaa\" \"aaa\")", "T");
 make_test ("(string= \"aaa\" \"aab\")", "()");
 make_test ("(string/= \"aaa\" \"aaa\")", "()");
 make_test ("(string/= \"aaa\" \"aab\")", "T");
+make_test ("(char= #\\a)", "T");
+make_test ("(char= #\\a #\\b)", "()");
+make_test ("(char= #\\a #\\a #\\a)", "T");
+make_test ("(char= #\\a #\\a #\\b)", "()");
+make_test ("(char-equal #\\a)", "T");
+make_test ("(char-equal #\\a #\\a)", "T");
+make_test ("(char-equal #\\a #\\A)", "T");
+make_test ("(char-equal #\\a #\\A #\\a)", "T");
+make_test ("(char-equal #\\a #\\A #\\b)", "()");
+make_test ("(char-upcase #\\a)", "#\\A");
+make_test ("(char-upcase #\\A)", "#\\A");
+make_test ("(char-upcase #\\3)", "#\\3");
+make_test ("(char-downcase #\\B)", "#\\b");
+make_test ("(char-downcase #\\b)", "#\\b");
+make_test ("(char-downcase #\\@)", "#\\@");
+make_test ("(alpha-char-p #\\a)", "T");
+make_test ("(alpha-char-p #\\3)", "()");
+make_test ("(alphanumericp #\\a)", "T");
+make_test ("(alphanumericp #\\3)", "T");
+make_test ("(alphanumericp #\\#)", "()");
+make_test ("(digit-char 2)", "#\\2");
+make_test ("(digit-char 10)", "()");
+make_test ("(digit-char 10 16)", "#\\A");
+make_test ("(digit-char 16 16)", "()");
+make_test ("(digit-char-p #\\2)", "2");
+make_test ("(digit-char-p #\\A)", "()");
+make_test ("(digit-char-p #\\a)", "()");
+make_test ("(digit-char-p #\\A 16)", "10");
+make_test ("(digit-char-p #\\a 16)", "10");
+make_test ("(digit-char-p #\\g 16)", "()");
+make_test ("(string-upcase \"Hello\")", "\"HELLO\"");
+make_test ("(string-downcase \"Hello\")", "\"hello\"");
+make_test ("(string-capitalize \"this is a Good day\")", "\"This Is A Good Day\"");
 make_test ("(car '(1 2))", "1");
 make_test ("(cdr '(1 2))", "(2)");
 make_test ("(car ())", "()");
@@ -354,17 +416,40 @@ make_test ("(nth-value 3 (values 10 9))", "()");
 make_test ("(elt '(0 3) 1)", "3");
 make_test ("(elt \"abc\" 0)", "#\\a");
 make_test ("(elt #(3 2 1) 1)", "2");
+make_test ("(aref \"abc\" 2)", "#\\c");
 make_test ("(aref #(1 2 3) 1)", "2");
+make_test ("(aref (make-array nil))", "()");
+make_test ("(aref (make-array '(1 2 3)) 0 1 2)", "()");
+make_test ("(row-major-aref #(0 1 2) 1)", "1");
+make_test ("(row-major-aref (make-array '(3 2 1)) 5)", "()");
 make_test ("(char \"àbcdef\" 2)", "#\\c");
 make_test ("(length ())", "0");
 make_test ("(length \"aaa\")", "3");
 make_test ("(length #(1 2))", "2");
 make_test ("(length '(0 1 2))", "3");
 make_test ("(list-length '(0 1 2 3))", "4");
+make_test ("(make-array 4)", "#(() () () ())");
+make_test ("(make-array '(1 2 3))", "#<ARRAY, RANK 3>");
+make_test ("(make-array nil)", "#<ARRAY, RANK 0>");
+make_test ("(array-has-fill-pointer-p \"aaa\")", "()");
+make_test ("(array-rank #())", "1");
+make_test ("(array-rank (make-array nil))", "0");
+make_test ("(array-rank (make-array '(1 2)))", "2");
 make_test ("(array-dimensions \"aaa\")", "(3)");
 make_test ("(array-dimensions #(1 2 3 4))", "(4)");
 make_test ("(array-dimension \"aaa\" 0)", "3");
 make_test ("(array-total-size \"aaa\")", "3");
+make_test ("(array-in-bounds-p (make-array nil))", "T");
+make_test ("(array-in-bounds-p \"aaa\" 3)", "()");
+make_test ("(array-in-bounds-p (make-array '(1 2 3)) 0 1 2)", "T");
+make_test ("(array-in-bounds-p (make-array '(1 2 3)) 1 1 2)", "()");
+make_test ("(array-row-major-index \"aaa\" 1)", "1");
+make_test ("(array-row-major-index #(0 1 2) 0)", "0");
+make_test ("(array-row-major-index (make-array nil))", "0");
+make_test ("(array-row-major-index (make-array '(1 2 3)) 0 0 0)", "0");
+make_test ("(array-row-major-index (make-array '(2 3 4)) 0 0 1)", "1");
+make_test ("(array-row-major-index (make-array '(2 3 4)) 0 2 0)", "8");
+make_test ("(array-row-major-index (make-array '(2 3 4)) 1 0 0)", "12");
 make_test ("(last '(1 2 3))", "(3)");
 make_test ("(last '(1 2 3) 0)", "()");
 make_test ("(last '(1 2 3) 2)", "(2 3)");
@@ -380,6 +465,18 @@ make_test ("(write-char #\\newline)", "\n#\\Newline");
 make_test ("(fresh-line)", "()");
 make_test ("(terpri)", "\n()");
 make_test ("(write-line \"aaa\")", "aaa\n\"aaa\"");
+make_test ("(prin1 '|Aa,a|)", "|Aa,a|\n|Aa,a|");
+make_test ("(prin1 #\\a)", "#\\a\n#\\a");
+make_test ("(prin1 \"aaa\")", "\"aaa\"\n\"aaa\"");
+make_test ("(princ '|Aa,a|)", "Aa,a\n|Aa,a|");
+make_test ("(princ #\\a)", "a\n#\\a");
+make_test ("(princ \"aaa\")", "aaa\n\"aaa\"");
+make_test ("(format t \"aaa\")", "aaa\n()");
+make_test ("(format t \"aaa~~\")", "aaa~\n()");
+make_test ("(format t \"aaa~%~%\")", "aaa\n\n()");
+make_test ("(format t \"aa~&~&bb\")", "aa\nbb\n()");
+make_test ("(format t \"the number is ~a and the list is ~s\" 10 '(1 2 3))",
+	   "the number is 10 and the list is (1 2 3)\n()");
 make_test ("(second '(0))", "()");
 make_test ("(fifth '(0 1 2 3 4))", "4");
 make_test ("(endp '(1 . 2))", "()");
@@ -407,6 +504,7 @@ make_test ("(cadddr '(0 1 2 3 4))", "3");
 make_test ("(caar '((4) 1 2 3))", "4");
 make_test ("(incf c)", "6");
 make_test ("(incf c 3)", "9");
+make_test ("(let ((i 0)) (incf i))", "1");
 make_test ("(decf c)", "8");
 make_test ("(decf c c)", "0");
 make_test ("(and)", "T");
@@ -436,6 +534,8 @@ make_test ("(reverse #(0 1 2 3))", "#(3 2 1 0)");
 make_test ("(nreverse '(0 1 2 3))", "(3 2 1 0)");
 make_test ("(nreverse \"hello\")", "\"olleh\"");
 make_test ("(nreverse #(0 1 2 3))", "#(3 2 1 0)");
+make_test ("(adjoin 0 '(0 1 2))", "(0 1 2)");
+make_test ("(adjoin 0 '(1 2))", "(0 1 2)");
 make_test ("(cond (t (+ 1 2)))", "3");
 make_test ("(cond (nil 2) (t 3) (nil 4))", "3");
 make_test ("(every #'evenp '(0 2 4))", "T");
